
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.042.
use strict;
use warnings;

use Module::Build 0.28;
use lib qw{inc}; use GPD::Build;

my %module_build_args = (
  "build_requires" => {
    "ExtUtils::ParseXS" => "2.30",
    "ExtUtils::Typemaps::Default" => "1.05",
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Alien::ProtoBuf" => "0.01",
    "Alien::uPB" => "0.02",
    "Module::Build" => "0.28",
    "Module::Build::WithXSpp" => "0.14"
  },
  "dist_abstract" => "fast and complete protocol buffer implementation",
  "dist_author" => [
    "Mattia Barbon <mattia\@barbon.org>"
  ],
  "dist_name" => "Google-ProtocolBuffers-Dynamic",
  "dist_version" => "0.04",
  "license" => "perl",
  "module_name" => "Google::ProtocolBuffers::Dynamic",
  "recursive_test_files" => 1,
  "requires" => {
    "perl" => "5.012"
  }
);


my %fallback_build_requires = (
  "ExtUtils::ParseXS" => "2.30",
  "ExtUtils::Typemaps::Default" => "1.05",
  "Module::Build" => "0.28"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = GPD::Build->new(%module_build_args);


$build->create_build_script;
