syntax = "proto2";

package test;

message MessageBefore {
    enum EnumBefore {
        FIRST = 1;
        SECOND = 2;
    }
    optional EnumBefore value = 1;
    repeated EnumBefore array = 2;
}

message MessageAfter {
    enum EnumAfter {
        FIRST = 1;
        SECOND = 2;
        THIRD = 3;
    }
    optional EnumAfter value = 1;
    repeated EnumAfter array = 2;
}
